@@ -1,3 +1,4 @@
+# pylint: disable=consider-using-f-string
 """Wrapper over the FTrack API to make it more pythonic to use.
 Supports the querying and creation of objects, and a way to build
 event listeners.
@@ -6,15 +7,18 @@ It's designed to hide the SQL-like syntax in favour of an object
 orientated approach. Inspiration was taken from SQLALchemy.
 """
 
-__all__ = ['FTrackQuery', 'entity', 'and_', 'or_', 'not_', 'event']
+__all__ = ['FTrackQuery', 'entity', 'and_', 'or_', 'not_', 'event',
+           'select', 'create', 'update', 'delete']
 
-__version__ = '1.6.4'
+__version__ = '1.7.0'
 
 import ftrack_api
 
+from . import utils
+from .abstract import AbstractStatement
 from .query import Query, entity, and_, or_, not_
 from .event import event
-from .utils import logger, dict_to_str
+from .statement import select, create, update, delete
 
 
 class FTrackQuery(ftrack_api.Session):
@@ -26,7 +30,7 @@ class FTrackQuery(ftrack_api.Session):
         If the debug argument is set, the connection will be ignored.
         """
         self.debug = kwargs.pop('debug', False)
-        self._logger = kwargs.pop('logger', logger)
+        self._logger = kwargs.pop('logger', utils.logger)
         self._logger.debug('Connecting...')
         if not self.debug:
             super(FTrackQuery, self).__init__(**kwargs)
@@ -40,7 +44,7 @@ class FTrackQuery(ftrack_api.Session):
             return super(FTrackQuery, self).__getattribute__(attr)
         except AttributeError:
             if self.debug or attr in super(FTrackQuery, self).__getattribute__('types'):
-                return Query.new(self, attr)
+                return Query(self, attr)
             raise
 
     def close(self, *args, **kwargs):
@@ -48,7 +52,7 @@ class FTrackQuery(ftrack_api.Session):
         if not self.debug:
             return super(FTrackQuery, self).close(*args, **kwargs)
 
-    def get(self, value, _value=None, *args, **kwargs):
+    def get(self, value, _value=None, *args, **kwargs):  # pylint: disable=keyword-arg-before-vararg
         """Get any entity from its ID.
         The _value argument is for compatibility with ftrack_api.Session.
         """
@@ -56,24 +60,24 @@ class FTrackQuery(ftrack_api.Session):
             entity = 'Context'
         else:
             entity, value = value, _value
-        self._logger.debug('Get: {}({})'.format(entity, value.__repr__()))
+        self._logger.debug('Get: %s(%r)', entity, value)
         return super(FTrackQuery, self).get(entity, value, *args, **kwargs)
 
     def query(self, query, *args, **kwargs):
         """Create an FTrack query object from a string."""
         query = str(query)
-        self._logger.debug('Query: '+query)
+        self._logger.debug('Query: %s', query)
         return super(FTrackQuery, self).query(query, *args, **kwargs)
 
     def create(self, entity, data, *args, **kwargs):
         """Create a new entity."""
         if not kwargs.get('reconstructing', False):
-            self._logger.debug('Create: {}({})'.format(entity, dict_to_str(data)))
+            self._logger.debug('Create: %s(%s)', entity, utils.dict_to_str(data))
         return super(FTrackQuery, self).create(entity, data, *args, **kwargs)
 
     def delete(self, entity, *args, **kwargs):
         """Delete an FTrack entity."""
-        self._logger.debug('Delete: '+entity.__repr__())
+        self._logger.debug('Delete: %r', entity)
         return super(FTrackQuery, self).delete(entity, *args, **kwargs)
 
     def where(self, *args, **kwargs):
@@ -95,3 +99,9 @@ class FTrackQuery(ftrack_api.Session):
         if isinstance(projections, (list, tuple, set)):
             projections = ','.join(map(str, projections))
         return super(FTrackQuery, self).populate(entities, projections)
+
+    def execute(self, stmt):
+        """Execute a statement."""
+        if isinstance(stmt, AbstractStatement):
+            return stmt.execute(self)
+        raise NotImplementedError(type(stmt))
