@@ -1,8 +1,27 @@
+# pylint: disable=consider-using-f-string
+"""Base query syntax.
+The classes in this file allow construction of a string to pass to the
+`ftrack_api.Session.query` method.
+
+Example:
+    >>> query = (
+    ...     session.Shot
+    ...     .where(
+    ...         ~entity.children.any(entity.name.like('%Animation%')),
+    ...         name='Shot 1',
+    ...     )
+    ...     .order_by(entity.created_at.desc())
+    ...     .limit(5)
+    ... )
+    >>> len(query.all())
+    5
+"""
+
 __all__ = ['entity', 'and_', 'or_', 'not_']
 
 import ftrack_api
 
-from .abstract import AbstractComparison, AbstractQuery
+from .abstract import AbstractComparison, AbstractQuery, AbstractStatement
 from .utils import Join, clone_instance, convert_output_value, not_, parse_operators
 
 
@@ -124,6 +143,25 @@ class Comparison(AbstractComparison):
         """
         if not args:
             return self.__class__('{} in ()'.format(self.value))
+        single_arg = len(args) == 1
+
+        # Args were given as a built query
+        # If a single query, then a subquery will work as long as a select is done
+        # If multiple queries, then raise an error
+        if single_arg and isinstance(args[0], AbstractQuery):
+            subquery = args[0].as_str()
+
+            # If a subquery has no projection, then select ID and remap the value
+            # eg. `parent in (Task)` will become `parent.id in (select id from Task)`
+            if not subquery.startswith('select '):
+                return self.__class__('{}.id in ({})'.format(
+                    self.value, args[0].select('id').as_str()
+                ))
+
+            args = [subquery]
+
+        elif isinstance(args[0], AbstractQuery):
+            raise ValueError('unable to check against multiple subqueries')
 
         # Args were given as entities
         if isinstance(args[0], ftrack_api.entity.base.Entity):
@@ -132,7 +170,7 @@ class Comparison(AbstractComparison):
             ))
 
         # Args were given as a list of strings
-        if len(args) == 1 and args[0].startswith('select ') and ' from ' in args[0]:
+        if single_arg and args[0].startswith('select ') and ' from ' in args[0]:
             subquery = args[0]
         else:
             subquery = ', '.join(map(convert_output_value, args))
@@ -146,11 +184,15 @@ class Comparison(AbstractComparison):
 
     def startswith(self, value):
         """If a value starts with this."""
-        return self.like(value.replace('%', '\\%') + '%')
+        return self.like(value.replace('%', '\\%') + '%')  # pylint: disable=no-value-for-parameter
 
     def endswith(self, value):
         """If a value ends with this."""
-        return self.like('%' + value.replace('%', '\\%'))
+        return self.like('%' + value.replace('%', '\\%'))  # pylint: disable=no-value-for-parameter
+
+    def contains(self, value):
+        """If a value contains this."""
+        return self.like('%' + value.replace('%', '\\%') + '%')  # pylint: disable=no-value-for-parameter
 
 
 class Query(AbstractQuery):
@@ -179,9 +221,9 @@ class Query(AbstractQuery):
     )
 
     def __init__(self, session, entity):
+        super(Query, self).__init__()
         self._session = session
         self._entity = entity
-        self._where = []
         self._populate = []
         self._sort = []
         self._offset = 0
@@ -200,8 +242,12 @@ class Query(AbstractQuery):
         """
         return Comparison(attr)
 
-    def __str__(self):
-        """Evaluate the query data and convert to a string."""
+    def __bool__(self):
+        return self._entity is not None
+    __nonzero__ = __bool__
+
+    def as_str(self):
+        """Generate a string from the query data."""
         query = []
         if self._populate:
             query.append('select')
@@ -222,6 +268,9 @@ class Query(AbstractQuery):
             query += ['limit', str(self._limit)]
         return ' '.join(filter(bool, query))
 
+    def __str__(self):
+        return self.as_str()
+
     def __call__(self, *args, **kwargs):
         """Custom error message if attempting to call.
         This is due to it being quite a common mistake.
@@ -245,7 +294,6 @@ class Query(AbstractQuery):
                         return result
                 raise
 
-
         raise TypeError("'Query' object is not callable, "
                         "perhaps you meant to use 'Query.where()'?")
 
@@ -259,22 +307,17 @@ class Query(AbstractQuery):
         self._where = [or_(and_(*self._where), and_(*entity._where))]
         return self
 
-    @classmethod
-    def new(cls, session, entity):
-        """Create a new Query object."""
-        return Query(session, entity)
-
     def copy(self):
         # pylint: disable=protected-access
         """Create a new copy of the class."""
-        cls = Query.new(session=self._session, entity=self._entity)
-        cls._entity = self._entity
-        cls._where = list(self._where)
-        cls._populate = list(self._populate)
-        cls._sort = list(self._sort)
-        cls._offset = self._offset
-        cls._limit = self._limit
-        return cls
+        new = type(self)(session=self._session, entity=self._entity)
+        new._entity = self._entity
+        new._where = list(self._where)
+        new._populate = list(self._populate)
+        new._sort = list(self._sort)
+        new._offset = self._offset
+        new._limit = self._limit
+        return new
 
     def get(self, value, _value=None):
         """Get an entity from the ID.
@@ -335,7 +378,10 @@ class Query(AbstractQuery):
 
         self._populate += map(str, args)
         return self
-    select = populate
+
+    def select(self, *args):
+        """Prefetch attributes with the query."""
+        return self.populate(*args)
 
     @clone_instance
     def sort(self, attribute=None):
@@ -347,9 +393,9 @@ class Query(AbstractQuery):
             attribute = str(attribute)
             if ' ' in attribute:
                 attribute, method = attribute.split(' ')
-                if method == 'descending':
+                if method in ('desc', 'descending'):
                     desc = True
-                elif method != 'ascending':
+                elif method != ('asc', 'ascending'):
                     raise NotImplementedError('unknown sorting method: {!r}'.format(method))
 
         if attribute is None:
@@ -357,7 +403,7 @@ class Query(AbstractQuery):
         else:
             self._sort.append((attribute, desc))
         return self
-    order = sort
+    order = order_by = sort
 
     @clone_instance
     def offset(self, value=None):
@@ -392,7 +438,7 @@ class Entity(object):
     """
 
     def __init__(self):
-        self._query = Query.new(None, None)
+        self._query = Query(None, None)
 
     def __getattr__(self, attr):
         """Bypass the methods of Query to just get attributes."""
